@@ -21,7 +21,7 @@ import traceback
 import pickle
 
 
-__version__ = "1.1.2"
+__version__ = "1.1.3"
 
 
 def main():
@@ -200,8 +200,10 @@ def main():
             elif config.config['fit_type'] == 'mh' or config.config['fit_type'] == 'pt':
                 # Note: mh vs pt difference is handled in Config by setting or not setting the exchange_every key.
                 alg = algs.BasicBayesMCMCAlgorithm(config)
+            elif config.config['fit_type'] == 'ym':
+                alg = algs.Yens_MCMC(config)
             elif config.config['fit_type'] == 'sa':
-                alg = algs.BasicBayesMCMCAlgorithm(config, sa=True)
+                alg = algs.BasicBayesMCMCAlgorithm(config, sa=True)    
             elif config.config['fit_type'] == 'sim':
                 alg = algs.SimplexAlgorithm(config)
             elif config.config['fit_type'] == 'ade':
